;Conditional assembly directives
USER_VERSION    .set   	1                       ;Set to 1 if for User version
                                                ;Set to 0 if for PE version
;*****************************************************************************
; File Name:	SFLSH_X8.ASM  
;
; Project:	F243 Serial Boot loader
; Originator:	TMS320F24x Applications Group, Texas Instruments.
;
; Target Sys:	F241/3 with Serial Communication Hardware.
;
; Description:	Serial Boot loader using an Intelligent Windows based PC Host
;		communicating via the SCI on the target F243.
;
;		This is the 241/3 version of the Serial Boot Loader.
;		This has been derived from the 'F240 Serial Boot Loader.
;	
;Status:	Release 2.12
;		
;						
;
;
; Last Update:	August 1, 1999
;
;*****************************************************************************
;

;-----------------------------------------------------------------------------
; Debug directives
;-----------------------------------------------------------------------------
		.def	GPR0		;General purpose registers.
		.def	GPR1
		.def	GPR2
		.def	GPR3
		.def	data_buf
		.def	length
		.def	dest_addr
		.def	B2_0
		.def	B2_1
		.def	B2_2
		.def	B2_3
		.def	B2_4
		.def	B2_5
		.def	B2_6
		.def	SPAD1
		.def	SPAD2

		.def	DUMMY
		.def	DLY10
		.def	DLY100
		.def	DLY3K3
		.def	SEG_ST
		.def	SEG_END
		.def	PROTECT

		.def	FL_ADRS
		.def	FL_DATA
		.def	ERROR

;Variables for ERASE and CLEAR
		.def	RPG_CNT
		.def	FL_ST
		.def	FL_END

;Variables for PROGRAM
		.def	PRG_paddr
		.def	PRG_length
		.def	PRG_bufaddr
		.def	BYTE_MASK

		.def	VBR_CNTR
		.def	DELAY
		.def	CHAR_RETRY_CNTR
		.def	ABSOLUTE_CHAR_CNT


                .include        x24x.h

;Miscellaneous
mSEC_K	.set	1000		;Delay constant
BUF_SADDR       .set    0328h           ;Start address for Data buffer
WSGR		.set	0FFFFh
VBR_MAX 	.set	09h		;# times valid char needs to be received
CRC_MAX		.set	03h		;# retries at each PLL setting before giving up.
B0_SADDR        .set    00200h          ;Block B0 start address
USER_START	.set    0040h


;-----------------------------------------------------------------------------
; Variable Declarations for on chip RAM Blocks
;-----------------------------------------------------------------------------
		.bss	GPR0,1		;General purpose registers.
		.bss	GPR1,1
		.bss	GPR2,1
		.bss	GPR3,1
		.bss	stk0,1
		.bss	stk1,1
		.bss	data_buf,1
		.bss	length,1
		.bss	dest_addr,1
		.bss	PRG_paddr,1
		.bss	PRG_length,1
		.bss	PRG_bufaddr,1
		.bss	BYTE_MASK,1

		.bss	B2_0,1
		.bss	B2_1,1
		.bss	B2_2,1
		.bss	B2_3,1
		.bss	B2_4,1
		.bss	B2_5,1
		.bss	B2_6,1
		.bss	SPAD1,1
		.bss	SPAD2,1

		.bss	DUMMY,1
		.bss	DLY10,1
		.bss	DLY100,1
		.bss	DLY3K3,1
		.bss	SEG_ST,1
		.bss	SEG_END,1
		.bss	PROTECT,1

		.bss	FL_ADRS,1	;Flash load address.
		.bss	FL_DATA,1	;Flash load data.
		.bss	ERROR,1 	;Error flag register.

;Variables for ERASE and CLEAR
		.bss	RPG_CNT,1	;Program pulse count.
		.bss	FL_ST,1 	;Flash start addr/Seg Cntrl Reg.
		.bss	FL_END,1	;Flash end address.

;Misc variables
		.bss	VBR_CNTR,1
		.bss	DELAY,1
		.bss	CHAR_RETRY_CNTR,1
		.bss	ABSOLUTE_CHAR_CNT,1

;-----------------------------------------------------------------------------
; M A C R O - Definitions
;-----------------------------------------------------------------------------
SBIT0		.macro	DMA, MASK	;Clear bit Macro
		LACC	DMA
		AND	#(0FFFFh-MASK)
		SACL	DMA
		.endm

SBIT1		.macro	DMA, MASK	;Set bit Macro
		LACC	DMA
		OR	#MASK
		SACL	DMA
		.endm


KICK_DOG	.macro			;Watchdog reset macro
		LDP	#00E0h
                SPLK    #05555h, WDKEY
                SPLK    #0AAAAh, WDKEY
		LDP	#0h
                .endm



POINT_0 	.macro
		LDP	#00h
		.endm

POINT_B0	.macro
		LDP	#04h
		.endm

POINT_B1	.macro
		LDP	#06h
		.endm

POINT_PF1	.macro
		LDP	#0E0h
		.endm

;==============================================================================
; Start here after Reset.
;==============================================================================
		.text

START:
		POINT_0

		.if (USER_VERSION)
		BCND	USER_START, BIO 	;If BIO=0 go to User's code
		.endif				;Else continue with Flsh prg.


		SETC	INTM			;Disable interrupts
		SPLK	#0h, IMR		;Mask all Ints
		SPLK	#0FFh, IFR		;Clear all Int flags
		CLRC	SXM			;Clear Sign Extension Mode
		CLRC	OVM			;Reset Overflow Mode
		CLRC	CNF			;Config Block B0 to Data
		SETC	XF			;set XF Hi (i.e. assert CTS on EVM )


		LDP	#WDCR>>7
		SPLK	#006Fh,WDCR		;Disable WD if VCCP=5V
		KICK_DOG

;===========================================================================
;Uart Initialisation
;===========================================================================
UART_INIT:	POINT_B1
		SPLK	#0h, CHAR_RETRY_CNTR	;Clear retry counter
		SPLK	#0h, ABSOLUTE_CHAR_CNT	;TEST ONLY
		SPLK	#0FFFFh, DELAY
	
;==============================================================================
;SCI Initialization 
;==============================================================================
SCI_INIT:       LDP     #OCRA>>7
		SPLK	#0FFFFh,OCRA		; Set up pins as SCI pins.
		SPLK	#0FEF3h,OCRB

		LDP	#SCICCR>>7              ;1 stop bit,no parity,8 char bits,
                SPLK    #0007h, SCICCR          ;async mode, idle-line protocol
                SPLK    #0013h, SCICTL1         ;Enable TX, RX, internal SCICLK,
                SPLK    #0000H, SCICTL2         ;Disable RX Int, TX Int.
                SPLK    #0000h, SCIHBAUD
                SPLK    #0065, SCILBAUD        ;Baud Rate=1200 b/s (10 MHz SYSCLK)


                SPLK    #0000H, SCIPRI          
                SPLK    #0023h, SCICTL1         ;Relinquish SCI from Reset.

CLR_VBR_CNTR:	POINT_B1
		SPLK	#0, VBR_CNTR		;Clear valid baud rate counter

UI01:		LDP 	#SCIRXST>>7
           	BIT     SCIRXST,BIT6		;Test RXRDY bit
                BCND    UI01,NTC		;If RXRDY=0,then repeat loop

	;Check if Char is as expected
		LACC	SCIRXBUF
CHECK_CHAR:	AND	#0FFh			;Clear upper byte
		SUB	#00Dh			;Compare with "CR"
		BCND	CLR_VBR_CNTR, NEQ
		LDP 	#0

INC_VBRC	POINT_B1
		LACC	VBR_CNTR		;Inc VBR counter
		ADD	#1h
		SACL	VBR_CNTR
		SUB	#VBR_MAX		;Is VBR counter > max value ?
		BCND	UI01, NEQ		;No! fetch another char

SND_ECHO	LACC	#0AAh			;Yes!
		LDP	#SCITXBUF>>7
		SACL	SCITXBUF		;Indicate to Host Baudrate locked
		
		B	FLSH_INIT

		
;===========================================================================
;Init Flash parameters
;===========================================================================
FLSH_INIT	POINT_B1
;		SPLK	#0200, DLY10		;Delay val for 10uS
;		SPLK	#2000, DLY100		;Delay val for 100uS
;		SPLK	#0FFFFh, DLY3K3 	;Dealy val for 3.3mS

		SPLK	#0h, SEG_ST		;Start addr of Flash array
		SPLK	#1FFFh, SEG_END 	;End Addr of Flash array
		SPLK	#0FF00h, PROTECT	;Enable all segments
		SPLK	#0h, ERROR		;Clear error flag
		SPLK	#0h, FL_ST		;Select Flash array 0

		

		B	XFER_KERNEL


               
;===========================================================================
;Transfer Comms Kernel + sequencer to B0 & set CNF=1 (i.e. B0 = FE00)
;===========================================================================
XFER_KERNEL	MAR	*, AR1
		LAR	AR1, #B0_SADDR
		LACC	#kernel_end
		SUB	#kernel_strt
		SACL	GPR1
		LACC	#kernel_strt
		RPT	GPR1
		TBLR	*+

		SETC	CNF
		
		B	MAIN




;===========================================================================
;M A I N   P R O G R A M 
;===========================================================================
		.sect	"kernel"
		.label	kernel_strt
MAIN:		
	;Load & Execute CLEAR
M00		CALL	FETCH_HEADER
		CALL	XFER_SCI_2_PROG


		CALL	ALGO_START
		LACC	ERROR
		CALL	SEND_CHAR	;Indicate to host Clear finished.

	;Load & Execute ERASE
M01		CALL	FETCH_HEADER
		CALL	XFER_SCI_2_PROG
		CALL	ALGO_START
		LACC	ERROR
		CALL	SEND_CHAR	;Indicate to host Erase finished.
		
	;Load & Execute PROG
M02		CALL	FETCH_HEADER
		CALL	XFER_SCI_2_PROG

M03		CALL	FETCH_HEADER		;Get info on Data block
		LACC	dest_addr
		SACL	PRG_paddr		;Pass Flash dest addr
		LACC	length
		ADD	#01h			;adjust for actual length
		SACL	PRG_length		;Pass Data block length
		SPLK	#BUF_SADDR, PRG_bufaddr ;Pass Data buffer start addr

M04		CALL	XFER_SCI_2_DATA		;Transfer Data block to B1
		CALL	ALGO_START		;Execute PROG routine
		LACC	ERROR
		CALL	SEND_CHAR

		CALL	FETCH_SCI_WORD		;Check if more blocks to come.
		LACC	data_buf		;If non-zero, then loop
		BCND	M03, NEQ		;If zero then finish up.

		

DEND		B	DEND			;TEST ONLY

;===========================================================================
; Routine Name: F E T C H _ H E A D E R	       Routine Type: SR
;
;===========================================================================
FETCH_HEADER:	CALL	FETCH_SCI_WORD
		LACC	data_buf
		SACL	dest_addr
		CALL	FETCH_SCI_WORD
		LACC	data_buf
		SACL	length
		RET


;===========================================================================
; Routine Name: X F E R _ S C I _ 2 _ P R O G	       Routine Type: SR
;===========================================================================
XFER_SCI_2_PROG:
		MAR	*, AR0
		LAR	AR0, length
		LACC	#ALGO_START		;ACC=dest address

XSP0		CALL	FETCH_SCI_WORD
		TBLW	data_buf		;data_buff-->[ACC]
		ADD	#01h			;ACC++
		BANZ	XSP0			;loop "length" times
		RET
		
;===========================================================================
; Routine Name: X F E R _ S C I _ 2 _ D A T A	       Routine Type: SR
;===========================================================================
XFER_SCI_2_DATA:
		MAR	*, AR1
		LAR	AR0, length		;AR0 is loop counter
		LAR	AR1, #BUF_SADDR 	;Dest --> B1 RAM

XSD0		CALL	FETCH_SCI_WORD
		LACC	data_buf
		SACL	*+, AR0
		BANZ	XSD0, AR1
		RET


;===========================================================================
; Routine Name: F E T C H _ S C I _ W O R D	       Routine Type: SR
;
; Description: Version which expects Lo byte / Hi byte sequence from Host &
;              also echos byte
;===========================================================================
FETCH_SCI_WORD:	POINT_B1
		SACL	stk0
		LDP	#SCIRXST>>7
FSW0		BIT   SCIRXST,BIT6  		;Test RXRDY bit
		BCND  FSW0, NTC	      		;If RXRDY=0,then repeat loop 
		LACC	SCIRXBUF		;First byte is Lo byte
		SACL	SCITXBUF		;Echo byte back
		AND	#0FFh			;Clear upper byte
	
FSW1		BIT   SCIRXST,BIT6  		;Test RXRDY bit
		BCND  FSW1, NTC	    		;If RXRDY=0,then repeat loop
		NOP 
		ADD	SCIRXBUF,8		;Concatenate Hi byte to Lo
		SFL				;used because 7 is max in SACH
		SACH	SCITXBUF,7		;Echo byte back (after SFL 8)

		POINT_B1
		SFR				;restore ACC as before
		SACL	data_buf		;Save received word

		LACC	stk0
		RET


;==============================================================================
; Transmit char to host subroutine.
;==============================================================================
SEND_CHAR	LDP	#SCITXBUF>>7
		SACL	SCITXBUF		;Transmit byte to host.
		POINT_B1
		RET

;==============================================================================
;Down-loaded algorithms start here.
;==============================================================================
ALGO_START      .word	0	
		.label	kernel_end

